(function(){
  var DefaultHandler, Parser, events, fs, path, request, soupselect, url, util, winston, select, handler, parser, APPLICATION_NAME, simplify, create_dir, PageReference, Crawler, _ref, __indexOf = [].indexOf || function(x){
    var i = -1, l = this.length;
    while (++i < l) if (this.hasOwnProperty(i) && this[i] === x) return i;
    return -1;
  };
  _ref = require('htmlparser'), DefaultHandler = _ref.DefaultHandler, Parser = _ref.Parser;
  events = require('events');
  fs = require('fs');
  path = require('path');
  request = require('request');
  soupselect = require('soupselect');
  url = require('url');
  util = require('util');
  winston = require('winston');
  select = soupselect.select;
  handler = new DefaultHandler;
  parser = new Parser(handler);
  global.error = winston.error;
  global.info = winston.info;
  global.warn = winston.warn;
  APPLICATION_NAME = 'skrolr';
  simplify = function(_url){
    var o;
    o = url.parse(_url);
    return o.protocol + '//' + o.host + o.path;
  };
  create_dir = function(dir){
    var stats;
    if (path.existsSync(dir)) {
      stats = fs.lstatSync(dir);
      if (!stats.isDirectory()) {
        return error("Invalid cache directory " + dir);
      }
    } else {
      info("Creating cache directory " + dir);
      return fs.mkdirSync(dir);
    }
  };
  PageReference = (function(){
    PageReference.displayName = 'PageReference';
    var counter, prototype = PageReference.prototype, constructor = PageReference;
    counter = 0;
    function PageReference(){
      counter = counter + 1;
      this.id = counter;
      this.links = [];
      this.tries = 1;
      this.title = '';
      this.url = '';
    }
    prototype.add_link = function(ref){
      if (__indexOf.call(this.links, ref) < 0 && this.id !== ref.id) {
        return this.links.push(ref);
      }
    };
    return PageReference;
  }());
  Crawler = (function(_super){
    Crawler.displayName = 'Crawler';
    var prototype = __extends(Crawler, _super).prototype, constructor = Crawler;
    function Crawler(){
      Crawler.superclass.apply(this, arguments);
      this.max_depth = 5;
      this.start_urls = [];
      this.accepted_domains = [];
      this.max_retries = 3;
      this.retry_wait = 1000;
      this.processed = {};
      this.user_agent = 'SCHMÜTZÏ';
      this.to_download = 0;
      this.cache_directory = '.cache/';
      this.cache_path = path.join(process.env['HOME'], this.cache_directory, APPLICATION_NAME);
      create_dir(path.join(process.env['HOME'], this.cache_directory));
      create_dir(path.join(this.cache_path));
      info("Using cache folder " + this.cache_path);
    }
    prototype.add_url = (function(){
      function add_url(_url){
        var o, _ref;
        this.start_urls.push(_url);
        o = url.parse(_url);
        if (_ref = o.host, __indexOf.call(this.accepted_domains, _ref) < 0) {
          this.accepted_domains.push(o.host);
          this.to_download += 1;
        }
      }
      return add_url;
    }());
    prototype.add_urls = (function(){
      function add_urls(url_list){
        var u, _i, _len;
        for (_i = 0, _len = url_list.length; _i < _len; ++_i) {
          u = url_list[_i];
          this.add_url(u);
        }
      }
      return add_urls;
    }());
    prototype.fetch = (function(){
      function fetch(_url, depth){
        var o, cachefname, options, _this = this;
        depth == null && (depth = 0);
        o = url.parse(_url);
        cachefname = this.cache_path + '/' + (o.host + o.path).replace(/\//g, '.');
        options = {
          uri: _url,
          'User-Agent': 'MEUH' + this.user_agent
        };
        try {
          return fs.readFile(cachefname, 'utf-8', function(err, fdata){
            if (err) {
              info("*** Fetching " + _url + " " + _this.to_download);
              return request(options, function(err, response, body){
                if (err) {
                  if (_this.processed[_url].tries < _this.max_retries) {
                    _this.processed[_url].tries += 1;
                    setTimeout(function(){
                      return _this.fetch(_url, depth);
                    }, _this.retry_wait);
                    return;
                  } else {
                    _this.to_download -= 1;
                    info("*** Max retries reached " + _url);
                    return;
                  }
                }
                fs.writeFile(cachefname, body, 'utf-8', null);
                _this.to_download -= 1;
                return _this.parse(body, _url, depth + 1);
              });
            } else {
              _this.to_download -= 1;
              return _this.parse(fdata, _url, depth + 1);
            }
          });
        } catch (e) {
          return error(e);
        }
      }
      return fetch;
    }());
    prototype.parse = (function(){
      function parse(body, base_url, depth){
        var query, i, m, _this = this;
        parser.parseComplete(body);
        info("*** Parsing " + base_url + " " + this.to_download);
        query = select(handler.dom, 'title');
        query.forEach(function(elt){
          var o;
          if (elt.hasOwnProperty('children')) {
            return _this.processed[base_url].title = elt.children[0].raw;
          } else {
            o = url.parse(base_url);
            return _this.processed[base_url].title = o.pathname;
          }
        });
        i = 0;
        m = select(handler.dom, 'a');
        m.forEach(function(elt){
          var newurl, o, ref, _ref;
          if (elt.hasOwnProperty('attribs')) {
            if (elt.attribs.hasOwnProperty('href') && elt.attribs.href != null) {
              newurl = url.resolve(base_url, elt.attribs.href);
              o = url.parse(newurl);
              if (_ref = o.host, __indexOf.call(_this.accepted_domains, _ref) >= 0) {
                newurl = simplify(newurl);
                if (!_this.processed.hasOwnProperty(newurl)) {
                  if (depth <= _this.max_depth) {
                    ref = new PageReference();
                    ref.url = newurl;
                    _this.processed[newurl] = ref;
                    _this.to_download += 1;
                    _this.fetch(newurl, depth);
                  } else {
                    return;
                  }
                } else {
                  ref = _this.processed[newurl];
                }
                return _this.processed[base_url].add_link(ref);
              }
            }
          }
        });
        if (this.to_download === 0) {
          info('*** Ending crawling');
          return this.emit('exit');
        }
      }
      return parse;
    }());
    prototype.start = (function(){
      function start(){
        var ref, _i, _ref, _len, _url;
        info('*** Starting crawling');
        for (_i = 0, _len = (_ref = this.start_urls).length; _i < _len; ++_i) {
          _url = _ref[_i];
          _url = simplify(_url);
          ref = new PageReference();
          ref.url = _url;
          this.processed[_url] = ref;
          this.fetch(_url);
        }
        return ref;
      }
      return start;
    }());
    return Crawler;
  }(events.EventEmitter));
  exports.Crawler = Crawler;
  function __extends(sub, sup){
    function ctor(){} ctor.prototype = (sub.superclass = sup).prototype;
    (sub.prototype = new ctor).constructor = sub;
    if (typeof sup.extended == 'function') sup.extended(sub);
    return sub;
  }
}).call(this);
